@@ -5,13 +5,13 @@
 #' @param num_results the number of results to return, max per page is 100
 #' @param label_id filter history only for this label
 #' @param page_token retrieve a specific page of results
-#' @inheritParams thread
-#' @references \url{https://developers.google.com/gmail/api/v1/reference/users/history/list}
+#' @inheritParams gm_thread
+#' @references <https://developers.google.com/gmail/api/v1/reference/users/history/list>
 #' @export
 #' @examples
 #' \dontrun{
 #' my_history = history("10")
 #' }
-history <- function(start_history_id = NULL, num_results = NULL, label_id = NULL, page_token = NULL,  user_id = "me"){
+gm_history <- function(start_history_id = NULL, num_results = NULL, label_id = NULL, page_token = NULL,  user_id = "me"){
   page_and_trim("history", user_id, num_results, label_id, start_history_id, page_token)
 }
